from lxml.etree import tostring
import html

class TXTML:

  @classmethod
  def _clean_text_(cls, text):
    return text.replace('\n', '')

  @classmethod
  def get_document_type(cls, document):
    return document.getchildren()[0].text

  @classmethod
  def get_HTML_from_document(cls, document):
    properties = {}

    while document.tag != 'text':
      properties[document.tag] = cls._clean_text_(document.text)
      document = document.getchildren()[0]

    return document, properties

  @classmethod
  def parse_full_10K(cls, doc):
    text = ""
    for child in doc.getchildren():
      if child.tag == 'sec-header':
          continue
      html, properties = TXTML.get_HTML_from_document(child)
      if properties['type'] == '10-K':
        text = text + html.text_content()
    return text

  @classmethod
  def to_xml(cls, doc):
    return html.unescape(tostring(doc).decode("utf8"))



from os.path import dirname, basename, isfile
import glob
from .edgar import Edgar
from .txtml import TXTML
from .company import Company
from .xbrl import XBRL, XBRLElement

__version__ = "4.1.1"

modules = glob.glob(dirname(__file__)+"/*.py")
__all__ = [ basename(f)[:-3] for f in modules if isfile(f) and not f.endswith('__init__.py')]



from typing import Dict, List
import re
from datetime import datetime
from lxml import etree

class XBRL(etree.ElementBase):
  CONTEXT_REF_NOT_DATE = ["us-gaap", "srt", "dei"]

  @classmethod
  def clean_tag(cls, elem):
    """
    Parse tag so 
      {http://fasb.org/us-gaap/2018-01-31}Assets
    becomes
      Assets
    """
    elem.tag = elem.tag[elem.tag.find("}")+1:]

  @classmethod
  def parse_context_ref(cls, context_ref):
    """
    Duration_1_1_2018_To_12_31_2018 becomes 2018-01-01 to 2018-12-31
    As_Of_12_31_2017 becomes 2017-12-31
    """
    context_ref_to_date_text = lambda s: datetime.strptime(s, "%m_%d_%Y").date().strftime("%Y-%m-%d")
    if context_ref.startswith("Duration"):
      if not any([val in context_ref for val in cls.CONTEXT_REF_NOT_DATE]):
        from_date = context_ref_to_date_text(context_ref[len("DURATION")+1:context_ref.find("_To_")])
        to_date = context_ref_to_date_text(context_ref[context_ref.find("_To_")+4:])
        return {"from": from_date, "to": to_date}
      else:
        from_date = context_ref_to_date_text(context_ref[len("DURATION")+1:context_ref.find("_To_")])
        end_idx = min([context_ref.find(val) for val in cls.CONTEXT_REF_NOT_DATE if context_ref.find(val) > -1])
        to_date = context_ref_to_date_text(context_ref[context_ref.find("_To_")+4:end_idx-1])
        return {"from": from_date, "to": to_date}

    elif context_ref.startswith("As_Of"):
      if not any([val in context_ref for val in cls.CONTEXT_REF_NOT_DATE]):
        return {"from": context_ref_to_date_text(context_ref[len("As_Of")+1:])}
      else:
        end_idx = min([context_ref.find(val) for val in cls.CONTEXT_REF_NOT_DATE if context_ref.find(val) > -1])
        from_date = context_ref_to_date_text(context_ref[len("As_Of")+1:end_idx-1])
        return {"from": from_date}

  @property
  def child(self):
    return self.getchildren()[0]

  @property
  def relevant_children(self):
    """
    Get children that are not `context`
    """
    return [child for child in self.child.getchildren() if not isinstance(child, etree._Comment) and "context" not in child.tag]

  @property
  def relevant_children_parsed(self):
    """
    Get children that are not `context`, `unit`, `schemaRef`
    """
    children = [child for child in self.child.getchildren() if not isinstance(child, etree._Comment) and "context" not in child.tag and "unit" not in child.tag and "schemaRef" not in child.tag]
    for elem in children:
      XBRL.clean_tag(elem)
    return children

class XBRLElement(etree.ElementBase):

  @property
  def child(self):
    return self.getchildren()[0]

  @property
  def attrib(self) -> Dict:
    return self.child.attrib

  @property
  def context_ref(self) -> Dict:
    return XBRL.parse_context_ref(self.attrib["contextRef"]) if self.attrib.get("contextRef") else {}

  @property
  def name(self):
    return ' '.join(re.findall('[A-Z][^A-Z]*', self.child.tag))

  @property
  def value(self) -> str:
    return self.child.text.replace("\n", "").strip()

  def to_dict(self) -> Dict:
    return {
      "name": self.name,
      "value": self.value,
      "context_ref": self.context_ref
    }



import requests
from lxml import html, etree

BASE_URL = "https://www.sec.gov"

class Company():

    def __init__(self, name, cik):
        self.name = name
        self.cik = cik
        self.url = f"https://www.sec.gov/cgi-bin/browse-edgar?action=getcompany&CIK={cik}"
        self._get_company_info()

    def _get_company_info(self):
        page = html.fromstring(requests.get(self.url).content)
        companyInfo = page.xpath("//div[@class='companyInfo']")[0]
        indentInfo = companyInfo.getchildren()[1]
        self.sic = indentInfo.getchildren()[1].text
        self.us_state = indentInfo.getchildren()[3].text

    def _get_filings_url(self, filing_type="", prior_to="", ownership="include", no_of_entries=100):
        url = self.url + "&type=" + filing_type + "&dateb=" + prior_to + "&owner=" +  ownership + "&count=" + str(no_of_entries)
        return url

    def get_all_filings(self, filing_type="", prior_to="", ownership="include", no_of_entries=100):
      url = self._get_filings_url(filing_type, prior_to, ownership, no_of_entries)
      page = requests.get(url)
      return html.fromstring(page.content)

    def _group_document_type(self, tree, document_type):
      result = []
      grouped = []
      for i, elem in enumerate(tree.xpath('//*[@id="seriesDiv"]/table/tr')):
        if i == 0:
          continue
        url = elem.xpath("td")[1].getchildren()[0].attrib["href"]
        grouped.append(url)
        if elem.xpath("td")[0].text == document_type:
          result.append(grouped)
          grouped = []
      return result

    def get_document_type_from_10K(self, document_type, no_of_documents=1):
      tree = self.get_all_filings(filing_type="10-K")
      url_groups = self._group_document_type(tree, "10-K")[:no_of_documents]
      result = []
      for url_group in url_groups:
        for url in url_group:
          url = BASE_URL + url
          content_page = get_request(url)
          table = content_page.find_class("tableFile")[0]
          for row in table.getchildren():
            if document_type in row.getchildren()[3].text:
              href = row.getchildren()[2].getchildren()[0].attrib["href"]
              href = BASE_URL + href
              doc = get_request(href)
              result.append(doc)
      return result

    def get_data_files_from_10K(self, document_type, no_of_documents=1, isxml=False):
      tree = self.get_all_filings(filing_type="10-K")
      url_groups = self._group_document_type(tree, "10-K")[:no_of_documents]
      result = []
      for url_group in url_groups:
        for url in url_group:
          url = BASE_URL + url
          content_page = get_request(url)
          table = content_page.find_class("tableFile")[1]
          for row in table.getchildren():
            if document_type in row.getchildren()[3].text:
              href = row.getchildren()[2].getchildren()[0].attrib["href"]
              href = BASE_URL + href
              doc = get_request(href, isxml=isxml)
              result.append(doc)
      return result

    def get_10Ks(self, no_of_documents=1):
      tree = self.get_all_filings(filing_type="10-K")
      elems = tree.xpath('//*[@id="documentsbutton"]')[:no_of_documents]
      result = []
      for elem in elems:
          url = BASE_URL + elem.attrib["href"]
          content_page = get_request(url)
          table = content_page.find_class("tableFile")[0]
          last_row = table.getchildren()[-1]
          href = last_row.getchildren()[2].getchildren()[0].attrib["href"]
          href = BASE_URL + href
          doc = get_request(href)
          result.append(doc)
      return result

    def get_10K(self):
      return self.get_10Ks(no_of_documents=1)[0]


def get_request(href, isxml=False):
    page = requests.get(href)
    if isxml:
      return etree.fromstring(page.content)
    else:
      return html.fromstring(page.content)

def get_documents(tree, no_of_documents=1):
    BASE_URL = "https://www.sec.gov"
    elems = tree.xpath('//*[@id="documentsbutton"]')[:no_of_documents]
    result = []
    for elem in elems:
        url = BASE_URL + elem.attrib["href"]
        content_page = get_request(url)
        url = BASE_URL + content_page.xpath('//*[@id="formDiv"]/div/table/tr[2]/td[3]/a')[0].attrib["href"]
        filing = get_request(url)
        result.append(filing)

    if len(result) == 1:
        return result[0]
    return result

def get_CIK_from_company(company_name):
    tree = get_request("https://www.sec.gov/cgi-bin/browse-edgar?company=" + company_name)
    CIKList = tree.xpath('//*[@id="seriesDiv"]/table/tr[*]/td[1]/a/text()')
    names_list = []
    for elem in tree.xpath('//*[@id="seriesDiv"]/table/tr[*]/td[2]'):
        names_list.append(elem.text_content())
    return list(zip(CIKList, namesList))



from lxml import html
import requests

class Edgar():

    def __init__(self):
        all_companies_page = requests.get("https://www.sec.gov/Archives/edgar/cik-lookup-data.txt")
        all_companies_content = all_companies_page.content.decode("latin1")
        all_companies_array = all_companies_content.split("\n")
        del all_companies_array[-1]
        all_companies_array_rev = []
        for i, item in enumerate(all_companies_array):
            if item == "":
                continue
            item_arr = item.split(":")
            all_companies_array[i] = (item_arr[0], item_arr[1])
            all_companies_array_rev.append((item_arr[1], item_arr[0]))
        self.all_companies_dict = dict(all_companies_array)
        self.all_companies_dict_rev = dict(all_companies_array_rev)

    def get_cik_by_company_name(self, name):
        return self.all_companies_dict[name]

    def get_company_name_by_cik(self, cik):
        return self.all_companies_dict_rev[cik]

    def find_company_name(self, words):
        possible_companies = []
        words = words.lower()
        for company in self.all_companies_dict:
            if all(word in company.lower() for word in words.split(" ")):
                possible_companies.append(company)
        return possible_companies
        
def test():
    com = Company("Oracle Corp", "0001341439")
    tree = com.get_all_filings(filingType = "10-K")
    return get_documents(tree)



from setuptools import setup, find_packages
from codecs import open
from os import path
import re

here = path.abspath(path.dirname(__file__))

version = re.search(
    '^__version__\s*=\s*"(.*)"',
    open('edgar/__init__.py').read(),
    re.M
    ).group(1)

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
        name = 'edgar',
        packages = find_packages(), # this must be the same as the name above
        version = version,
        description = 'Scrape data from SEC\'s EDGAR',
        long_description = long_description,
        author = 'Joey Sham',
        author_email = 'sham.joey@gmail.com',
        url = 'https://github.com/joeyism/py-edgar', # use the URL to the github repo
        download_url = 'https://github.com/joeyism/py-edgar/archive/{}.tar.gz'.format(version),
        keywords = ['edgar', 'sec'], 
        install_requires = ['requests', 'lxml'],
        classifiers = [],
        )
